/*
*  sound-plugin-hal-clovertrail
*
* Copyright (C) 2013  Intel Corporation. All rights reserved.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mm_error.h>
#include <mm_debug.h>
#include <mm_sound.h>
#include <mm_sound_plugin_hal.h>
#include <avsys-audio.h>

#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

#include "call_audio_clv.h"
#include "sound_card_control_clv.h"

#define DEVICE_API_BLUETOOTH    "bluez"
#define DEVICE_BUS_BLUETOOTH    "bluetooth"
#define DEVICE_BUS_USB          "usb"
#define DEVICE_API_ALSA         "alsa"
#define DEVICE_BUS_WIRED        "wired"
#define DEVICE_BUS_BUILTIN      "builtin"

#define DEVICE_API_MODEM        "alsa_modem"
#define DEVICE_BUS_MODEM        "modem"

static int  prev_mode     = MODE_NORMAL;
static int  current_mode  = MODE_NORMAL;

EXPORT_API
int MMSoundGetPluginType(void)
{
    return MM_SOUND_PLUGIN_TYPE_HAL;
}

static int hal_plug_set_sound_path(int gain, int output, int input, int option)
{
    int ret = AUDIO_HAL_STATE_SUCCESS;
    debug_fenter();
    debug_log ("[Input Param] gain %d, out %d, in %d, opt 0x%x", gain, output, input, option);
    /* Check the gain */
    switch (gain)
    {
    case AVSYS_AUDIO_GAIN_EX_KEYTONE:
    case AVSYS_AUDIO_GAIN_EX_ALARMTONE:
    case AVSYS_AUDIO_GAIN_EX_AUDIOPLAYER:
    case AVSYS_AUDIO_GAIN_EX_VIDEOPLAYER:
    case AVSYS_AUDIO_GAIN_EX_CAMERA:
    case AVSYS_AUDIO_GAIN_EX_GAME:
    case AVSYS_AUDIO_GAIN_EX_RINGTONE:
    case AVSYS_AUDIO_GAIN_EX_CALLTONE:
        /*if (current_mode == MODE_IN_CALL) {
            debug_log("CLV: current mode is MODE_IN_CALL, skip audio path set");
            return ret;
        }
         NORMAL mode
         Check dual_out option
        if (option & AVSYS_AUDIO_PATH_OPTION_DUAL_OUT) {
            debug_log("CLV: Set audio path to dual out");
            MMSoundMgrPulseSetDefaultSink (DEVICE_API_ALSA, DEVICE_BUS_WIRED);
            ret = set_dual_out();
            return ret;
           Check output device
        } else if (output == AVSYS_AUDIO_PATH_EX_HEADSET) {
            debug_log("CLV: Set pulse default sink to wired_accessary");
            MMSoundMgrPulseSetDefaultSink (DEVICE_API_ALSA, DEVICE_BUS_WIRED);
        } else if (output == AVSYS_AUDIO_PATH_EX_SPK) {
            debug_log("CLV: Set pulse default sink to speaker");
            MMSoundMgrPulseSetDefaultSink (DEVICE_API_ALSA, DEVICE_BUS_BUILTIN);
        }*/
        break;

    /* recorder */
    case AVSYS_AUDIO_GAIN_EX_VOICEREC:
    case AVSYS_AUDIO_GAIN_EX_CAMCORDER:
        break;

    case AVSYS_AUDIO_GAIN_EX_VOICECALL:
        /*debug_log("CLV: Set call audio sound path to 0x%x", output);
         release path
        if (output == AVSYS_AUDIO_PATH_EX_NONE && input == AVSYS_AUDIO_PATH_EX_NONE) {
            prev_mode = current_mode;
            current_mode = MODE_NORMAL;
        } else if (current_mode != MODE_IN_CALL) {
            prev_mode = current_mode;
            current_mode = MODE_IN_CALL;
            debug_log("CLV: Set pulse default sink to modem for call audio mixing");
            MMSoundMgrPulseSetDefaultSink (DEVICE_API_MODEM, DEVICE_BUS_MODEM);
        }
        debug_log("CLV: prev_mode=%d, cur_mode=%d", prev_mode, current_mode);
        if (current_mode == MODE_IN_CALL) {
            debug_log("CLV: telephony subsystem handles voice call sound path set, return");
            return ret;
        }*/
        break;

    case AVSYS_AUDIO_GAIN_EX_VIDEOCALL:
    case AVSYS_AUDIO_GAIN_EX_FMRADIO:
        break;
    }
    /* Set Path (GAIN, OUT, IN) */
    /*if (AVSYS_FAIL(avsys_audio_set_path_ex(gain, output, input, option))) {
        debug_error("avsys_audio_set_path_ex failed\n");
        ret = AUDIO_HAL_STATE_ERROR_INTERNAL;
    }*/
    debug_fleave();
    return ret;
}

static int hal_plug_pulse_sink_route(int device)
{
    debug_fenter();
    /* Pulseaudio sink route */
    /*if (device & MM_SOUND_DEVICE_OUT_BT_A2DP) {
        debug_log("CLV: BT A2DP is active, Set pulse default sink to BLUEZ");
        MMSoundMgrPulseSetDefaultSink (DEVICE_API_BLUETOOTH, DEVICE_BUS_BLUETOOTH);
        return AUDIO_HAL_ROUTE_SUCCESS_AND_GOTOEND;
    } else if (device & MM_SOUND_DEVICE_OUT_WFD) {
#ifdef USE_PULSE_WFD  Not enabled yet
        debug_log("CLV: WFD is active, Set pulse default sink to WFD");
        MMSoundMgrPulseSetDefaultSink (DEVICE_API_WFD, DEVICE_BUS_BUILTIN);
        return AUDIO_HAL_ROUTE_SUCCESS_AND_GOTOEND;
#endif
    } else if (device & MM_SOUND_DEVICE_OUT_USB_AUDIO){
        debug_log("CLV: USB Audio is active, Set default sink to USB Audio");
        MMSoundMgrPulseSetDefaultSink (DEVICE_API_ALSA, DEVICE_BUS_USB);
        return AUDIO_HAL_ROUTE_SUCCESS_AND_GOTOEND;
    }*/
    debug_fleave();
    return AUDIO_HAL_STATE_SUCCESS;
}

static int hal_plug_pulse_source_route(int device)
{
    return AUDIO_HAL_STATE_SUCCESS;
}

static int hal_plug_init()
{
    debug_fenter();

    sound_card_ctrl_set(71, 50);
    sound_card_ctrl_set(10, 194);
    sound_card_ctrl_set(100, 1);
    sound_card_ctrl_set(6, 180);
    sound_card_ctrl_set(63, 62);
    sound_card_ctrl_set(5, 108);
    sound_card_ctrl_set(4, 0);
    sound_card_ctrl_set(93, 1);
    sound_card_ctrl_set(94, 1);
    sound_card_ctrl_set(95, 1);
    sound_card_ctrl_set(96, 1);
    sound_card_ctrl_set(74, 1);
    sound_card_ctrl_set(79, 1);
    sound_card_ctrl_set(67, 61);

    sound_card_ctrl_set(72, 61);
    sound_card_ctrl_set(68, 62);
    sound_card_ctrl_set(53, 62);
    sound_card_ctrl_set(64, 62);
    sound_card_ctrl_set(57, 62);

    sound_card_ctrl_set(10, 215);
    sound_card_ctrl_set(6, 204);
//    lpe_setup_clv();
    debug_fleave();
    return AUDIO_HAL_STATE_SUCCESS;
}

static int hal_plug_fini()
{
    return AUDIO_HAL_STATE_SUCCESS;
}

EXPORT_API
int MMSoundPlugHALGetInterface(mmsound_hal_interface_t *intf)
{
    debug_fenter ();
    if(intf) {
        //intf->pulse_source_route = hal_plug_pulse_source_route;
        //intf->pulse_sink_route = hal_plug_pulse_sink_route;
        intf->set_sound_path = hal_plug_set_sound_path;
        intf->init=hal_plug_init;
        intf->fini=hal_plug_fini;
    }
    debug_fleave ();
    return AUDIO_HAL_STATE_SUCCESS;
}

